// test

#include "Arduino.h"
#include "Wire.h"
#include "ES8388.h"  // https://github.com/maditnerd/es8388
#include <driver/I2S.h>

// I2C GPIOs
#define IIC_CLK       32
#define IIC_DATA      33

// Amplifier enable
#define GPIO_PA_EN    21

#define key1 36
#define key2 13
#define key3 19
#define key4 23
#define key5 18
#define key6 5

ES8388 es;

#define buflen 180 // mono

const i2s_port_t I2S_PORT = I2S_NUM_0;
static const i2s_pin_config_t pin_config = {
    .bck_io_num   = 27,
    .ws_io_num    = 25,
    .data_out_num = 26,
    .data_in_num  = 35
};

// The I2S config as per the example
const i2s_config_t i2s_config = 
{
    .mode = i2s_mode_t(I2S_MODE_MASTER | I2S_MODE_RX | I2S_MODE_TX), 
    .sample_rate = 8000,                          // 8KHz
    .bits_per_sample = I2S_BITS_PER_SAMPLE_16BIT, // 
    .channel_format = I2S_CHANNEL_FMT_ONLY_RIGHT, // although the SEL config should be left, it seems to transmit on right
    //.channel_format = I2S_CHANNEL_FMT_RIGHT_LEFT,  // although the SEL config should be left, it seems to transmit on right
    .communication_format = I2S_COMM_FORMAT_I2S,
    .intr_alloc_flags = ESP_INTR_FLAG_LEVEL1,     // Interrupt level 1
    .dma_buf_count = 4,                           // number of buffers
    .dma_buf_len = buflen                         // 32 x 16 bit = 64 bytes
};

//----------------------------------------------------------------------------------------------------------------------

void setup()
{
    esp_err_t err;
    
    Serial.begin(250000);

    pinMode(key1, INPUT_PULLUP);
    pinMode(key2, INPUT_PULLUP);
    pinMode(key3, INPUT_PULLUP);
    pinMode(key4, INPUT_PULLUP);
    pinMode(key5, INPUT_PULLUP);
    pinMode(key6, INPUT_PULLUP);

    Serial.printf("Connect to ES8388 codec... ");
    while (not es.begin(IIC_DATA, IIC_CLK))
    {
        Serial.printf("Failed!\n");
        delay(1000);
    }
    Serial.printf("OK\n");

    es.volume(ES8388::ES_MAIN, 80);
    es.mute(ES8388::ES_MAIN, false);
  
   // Configuring the I2S driver and pins.
   // This function must be called before any I2S driver read/write operations.
   err = i2s_driver_install(I2S_PORT, &i2s_config, 0, NULL);
   if (err != ESP_OK) 
   {
     Serial.printf("Failed installing driver: %d\n", err);
     while (true);
   }

   err = i2s_set_pin(I2S_PORT, &pin_config);
   if (err != ESP_OK) 
   {
     Serial.printf("Failed setting pin: %d\n", err);
     while (true);
   }
   Serial.println("I2S driver installed.");
}
//----------------------------------------------------------------------------------------------------------------------


void loop()
{
    // Read a single sample and log it for the Serial Plotter.
    int16_t sample[buflen];
    int bytes_read;
    int bytes_write;

    bytes_read = i2s_read_bytes(I2S_PORT, (char *)&sample, 2*buflen, portMAX_DELAY);
    // 64bytes = 32samples in 4ms 
    if (bytes_read > 0) 
    {
      Serial.print("r ");
      Serial.print(bytes_read);
    }
    
    bytes_write = i2s_write_bytes(I2S_PORT, (char *)&sample, 2*buflen, portMAX_DELAY);
    // 64bytes = 32samples in 4ms 
    if (bytes_write > 0) 
    {
      Serial.print(" w ");
      Serial.print(bytes_write);
    }
    Serial.println();

    if (digitalRead(key1)==LOW) es.write_reg(ES8388_ADDR, ES8388_ADCCONTROL10,0xC0);
    if (digitalRead(key2)==LOW) es.write_reg(ES8388_ADDR, ES8388_ADCCONTROL10,0xD0);
    if (digitalRead(key3)==LOW) es.write_reg(ES8388_ADDR, ES8388_ADCCONTROL10,0xE0);
    if (digitalRead(key4)==LOW) es.write_reg(ES8388_ADDR, ES8388_ADCCONTROL10,0xE8);
    if (digitalRead(key5)==LOW) es.write_reg(ES8388_ADDR, ES8388_ADCCONTROL10,0xF0);
    if (digitalRead(key6)==LOW) es.write_reg(ES8388_ADDR, ES8388_ADCCONTROL10,0xF8);
}
//----------------------------------------------------------------------------------------------------------------------
